//
//  MIT License
//
//  Copyright (c) 2022 Tamerlan Satualdypov
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in all
//  copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.

import Foundation

/**
 Fetcher protocol that all fetchers conform to.
 
 If you want to create your own fetcher you must
 conform it to this protocol to be able to pass it
 to the `Snitch`.
 */
public protocol Fetcher {
    /**
     Title of the fetcher that will be shown in the
     Home Screen quick action menu.
     */
    var title: String { get }
    
    /**
     Icon of the fetcher that will be shown in the
     Home Screen quick action menu.
     */
    var icon: Icon? { get }
    
    /**
     Function that is called by the `Snitch` to
     retrieve the value from the fetchers.
     
     - Returns:
        Raw fetched data.
     */
    func fetch() -> String?
}
